/*
** Copyright (C) 1999 Eleftherios Gkioulekas <lf@amath.washington.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
# include <stdlib.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#else
# include <strings.h>
#endif
#include <mu.h>

/* Detect the endian-ness of the current platform */
int mu_endian_detect (void)
{
  union { unsigned int l; char c[sizeof(unsigned int)]; } u;
  u.l = 0x04030201L;
  return ((u.c[0]+'0' == '1') ? mu_file_little_endian : mu_file_big_endian);
}

/*
 * Endian and descriptive strings
 */

/* Check if this is a valid endian argument */
void mu_endian_validate (int endian, char *routine)
{
  if ((endian != mu_file_little_endian) &&
      (endian != mu_file_big_endian))
    {
      mu_error (mu_error_status, MU_EINVAL,
		"Invalid endian argument in %s", routine);
    }
}

/* Return a string describing the endianness of the system */
char * mu_endian_to_string (int endian)
{
  static char big[] = "big";
  static char little[] = "little";
  mu_endian_validate (endian, "mu_endian_to_string");
  return (endian == mu_file_little_endian) ? little : big;
}

/* Return the endian type described by the given string */
int mu_endian_from_string (char *str)
{
  if      (strcmp (str, "big") == 0) return mu_file_big_endian;
  else if (strcmp (str, "little") == 0) return mu_file_little_endian;
  else
    {
      mu_error (mu_error_status, MU_EINVAL,
		"Invalid string describing endianess: %s", str);
    }
}

/* Check if the string describing endian type is actually okey */
int mu_endian_valid_filetype_p (char *str)
{
  return ((strcmp (str, "big") == 0) || (strcmp (str, "little") == 0));
}

/* 
 * Endian routines for swaping
 */

void mu_endian_swap4 (void *p)
{
  static unsigned char out[4];
  unsigned char *c = (unsigned char *) p;
  out[3] = c[0]; 
  out[2] = c[1]; 
  out[1] = c[2]; 
  out[0] = c[3];
  c[0] = out[0]; 
  c[1] = out[1]; 
  c[2] = out[2]; 
  c[3] = out[3];
}

void mu_endian_swap8 (void *p)
{
  static unsigned char out[8];
  unsigned char *c = (unsigned char *) p;
  out[7] = c[0];
  out[6] = c[1];
  out[5] = c[2];
  out[4] = c[3];
  out[3] = c[4];
  out[2] = c[5];
  out[1] = c[6];
  out[0] = c[7];
  c[0] = out[0];
  c[1] = out[1];
  c[2] = out[2];
  c[3] = out[3];
  c[4] = out[4];
  c[5] = out[5];
  c[6] = out[6];
  c[7] = out[7];
}

void mu_endian_swap4_block (void *p, size_t len)
{
  int i;
  static unsigned char out[4];
  unsigned char *c = (unsigned char *) p;

  loop (i,0,len-1)
    {
      out[3] = c[0]; out[2] = c[1]; out[1] = c[2]; out[0] = c[3];
      c[0] = out[0]; c[1] = out[1]; c[2] = out[2]; c[3] = out[3];
      c += 4;
    }
}

void mu_endian_swap8_block (void *p, size_t len)
{
  int i;
  static unsigned char out[8];
  unsigned char *c = (unsigned char *) p;

  loop (i,0,len-1)
    {
      out[7] = c[0]; out[6] = c[1]; out[5] = c[2]; out[4] = c[3];
      out[3] = c[4]; out[2] = c[5]; out[1] = c[6]; out[0] = c[7];
      c[0] = out[0]; c[1] = out[1]; c[2] = out[2]; c[3] = out[3];
      c[4] = out[4]; c[5] = out[5]; c[6] = out[6]; c[7] = out[7];
      c = c + 8;
    }
}
