/*
** Copyright (C) 1999 Eleftherios Gkioulekas <lf@amath.washington.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
# include <stdlib.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#else
# include <strings.h>
#endif
#include <mu.h>

void mu_var_register (struct mu_var_handler_table *table,
		      mu_var_handler_t *handler)
{
  /* Find the tail of the list of tables */
  while (table->next != NULL) table = table->next;

  /* Now make a new entry */
  table->next = mu_malloc (1, sizeof (struct mu_var_handler_table));

  /* Record the handler on the new entry */
  table = (struct mu_var_handler_table *) table->next;
  table->handler = handler;
  table->next = NULL;
}

static char * mu_var_parse_format_string (char *fmt, char **fmtptr)
{
  static char *buffer = NULL;
  static size_t buffer_len = 0;
  char *p1, *p2;

  /* Find the beginning of the next substring by searching for the
     `%' character */
  p1 = strchr ((*fmtptr), '%');
  if (p1 == NULL) return NULL;
  if (*(p1+1) == '\0') 
    {
      mu_error (mu_error_status, MU_EINVAL, 
		"Stray character in format string `%s'" , fmt);
    }

  /* Set (p1,p2) equal to the next substring */
  p1 = strchr ((*fmtptr), '%');
  if (p1 == NULL) return NULL; /* If there are no more substrings, stop */
  p1 = p1 + 1;
  p2 = p1 + 1; while (*p2 != ' ' && *p2 != '%' && *p2 != '\0') p2++;
  p2--;

  /* Form the substring */
  if (buffer_len < p2 - p1 + 2)
    {
      buffer_len = p2 - p1 + 2;
      buffer = mu_realloc (buffer, buffer_len, sizeof (char));
    }
  strncpy (buffer, p1, p2 - p1 + 1);
  buffer[p2-p1+1] = '\0';

  /* Modify the pointer fmtptr and return the substring */
  *fmtptr = p2;
  return buffer;
}

void mu_var_call (struct mu_var_handler_table *table, void *clientdata,
		  char *routine, char *fmt, va_list *arg)
{
  char *fmtptr = fmt;
  char *spec;
  
  /* Loop over all the entries in the format string */
  while ((spec = mu_var_parse_format_string (fmt, &fmtptr)) != NULL)
    {
      struct mu_var_handler_table *search = table;
      /* Search for a handler that can handle this entry. 
	 When one of the handlers says that it worked, stop */
      while ((*(table->handler)) (clientdata, spec, arg) == 0)
	{
	  table = (struct mu_var_handler_table *) table->next;
	  if (table == NULL)
	    mu_error (mu_error_status, MU_EINVAL,
		      "Unrecognized spec in call to %s: %s", routine, spec);
	}
    }
}

void mu_var_test (char *fmt)
{
  char *fmtptr = fmt;
  char *str;
  while ((str = mu_var_parse_format_string (fmt, &fmtptr)) != NULL)
    {
      printf ("%s  -> %d \n", str, strlen (str));
    }
}
