/*
** Copyright (C) 1999 Eleftherios Gkioulekas <lf@amath.washington.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
# include <stdlib.h>
#endif
#if HAVE_MALLOC_H
# include <malloc.h>
#endif
#include <mu.h>
#include <assert.h>

typedef struct mu_double_array double_array;
typedef struct mu_double_matrix double_matrix;
typedef struct mu_double_cube double_cube;

void test_open_and_close (void)
{
  struct mu_file *file;
 
  file = mu_file_open ("test2_output.dat", mu_file_for_writing);
  mu_file_close (file);
  file = mu_file_open ("test2_output.dat", mu_file_for_reading);
  mu_file_close (file);
  file = mu_file_open ("test2_output.dat", mu_file_for_appending);
  mu_file_close (file);
  mu_delete ("test2_output.dat");
}

void test_simple (void)
{
  struct mu_file *file;
  double a = 12.2; double aa;
  float b = 32.4;  float bb;
  int c = 256;     int cc;

  /* Write contents of file */
  file = mu_file_open ("test2_output.dat", mu_file_for_writing);
  mu_file_write_double (file, &a, 1);
  mu_file_write_float  (file, &b, 1);
  mu_file_write_int    (file, &c, 1);
  mu_file_close        (file);

  /* Append more content to the file */
  file = mu_file_open ("test2_output.dat", mu_file_for_appending);
  mu_file_write_double (file, &a, 1);
  mu_file_write_float  (file, &b, 1);
  mu_file_write_int    (file, &c, 1);
  mu_file_close (file);

  /* Read the file and verify */
  file = mu_file_open ("test2_output.dat", mu_file_for_reading);
  aa = 0; bb = 0; cc = 0;
  mu_file_read_double (file, &aa, 1); assert (a == aa);
  mu_file_read_float  (file, &bb, 1); assert (b == bb);
  mu_file_read_int    (file, &cc, 1); assert (c == cc);
  aa = 0; bb = 0; cc = 0;
  mu_file_read_double (file, &aa, 1); assert (a == aa);
  mu_file_read_float  (file, &bb, 1); assert (b == bb);
  mu_file_read_int    (file, &cc, 1); assert (c == cc);
  mu_file_close (file);

  /* Done */
  mu_delete ("test2_output.dat");
}

void test_simple_variadic (void)
{
  struct mu_file *file;
  double a = 13.2; double aa;
  float b = 32.4;  float bb;
  int c = 256;     int cc;

  /* Write contents of file */
  file = mu_file_open ("test2_output.dat", mu_file_for_writing);
  mu_file_output (file, "%d %f %i", a, b, c);
  mu_file_close (file);

  /* Read the file and verify */
  file = mu_file_open ("test2_output.dat", mu_file_for_reading);
  mu_file_input (file, "%d %f %i", &aa, &bb, &cc);
  mu_file_close (file);

  /* Verify */
  mu_delete ("test2_output.dat");
  assert (a == aa);
  assert (b == bb);
  assert (c == cc);
} 

int main ()
{
  test_open_and_close ();
  test_simple (); 
  test_simple_variadic ();
}

