/*
** Copyright (C) 1999 Eleftherios Gkioulekas <lf@amath.washington.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#if HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdio.h>
#if STDC_HEADERS
# include <stdlib.h>
#endif
#if HAVE_MALLOC_H
# include <malloc.h>
#endif
#include <mu.h>
#include <assert.h>

const size_t ax = 100;
const size_t mx = 100;
const size_t my = 110;
const size_t cx = 40;
const size_t cy = 30;
const size_t cz = 35;

/*
 * Create test double arrays
 */

double_array_t test_make_double_array (void)
{
  int i;
  double_array_t a = mu_double_array_alloc (ax);
  loop_array (i,a) A_(a,i) = (double) i;
  return a;
}

double_matrix_t test_make_double_matrix (void)
{
  int i,j;
  double_matrix_t a = mu_double_matrix_alloc (mx, my);
  loop_matrix (i,j,a) M_(a,i,j) = (double) (i+j);
  return a;
}

double_cube_t test_make_double_cube (void)
{
  int i,j,k;
  double_cube_t a = mu_double_cube_alloc (cx, cy, cz);
  loop_cube (i,j,k,a) C_(a,i,j,k) = (double) (i+j+k);
  return a;
}

/*
 * Create test float arrays
 */

float_array_t test_make_float_array (void)
{
  int i;
  float_array_t a = mu_float_array_alloc (ax);
  loop_array (i,a) A_(a,i) = (float) i;
  return a;
}

float_matrix_t test_make_float_matrix (void)
{
  int i,j;
  float_matrix_t a = mu_float_matrix_alloc (mx, my);
  loop_matrix (i,j,a) M_(a,i,j) = (float) (i+j);
  return a;
}

float_cube_t test_make_float_cube (void)
{
  int i,j,k;
  float_cube_t a = mu_float_cube_alloc (cx, cy, cz);
  loop_cube (i,j,k,a) C_(a,i,j,k) = (float) (i+j+k);
  return a;
}

/*
 * Create test int arrays
 */

int_array_t test_make_int_array (void)
{
  int i;
  int_array_t a = mu_int_array_alloc (ax);
  loop_array (i,a) A_(a,i) = (int) i;
  return a;
}

int_matrix_t test_make_int_matrix (void)
{
  int i,j;
  int_matrix_t a = mu_int_matrix_alloc (mx, my);
  loop_matrix (i,j,a) M_(a,i,j) = (int) (i+j);
  return a;
}

int_cube_t test_make_int_cube (void)
{
  int i,j,k;
  int_cube_t a = mu_int_cube_alloc (cx, cy, cz);
  loop_cube (i,j,k,a) C_(a,i,j,k) = (int) (i+j+k);
  return a;
}

/* Only necessary to test one type. All other types are similar */
void test_simple (void)
{
  char *filename = "test3_output.dat";
  struct mu_file *file;
  double_array_t a1,a2;
  double_matrix_t m1,m2;
  double_cube_t c1,c2;
  int i,j,k;

  /* Create the arrays */
  a1 = test_make_double_array (); 
  m1 = test_make_double_matrix ();
  c1 = test_make_double_cube ();
  a2 = mu_double_array_alloc (ax);
  m2 = mu_double_matrix_alloc (mx, my);
  c2 = mu_double_cube_alloc (cx, cy, cz);
  
  /* Create the test file */
  file = mu_file_open (filename, mu_file_for_writing);
  mu_double_array_write (file, a1);
  mu_double_matrix_write (file, m1);
  mu_double_cube_write (file, c1);
  mu_file_close (file);

  /* Read the test file */
  file = mu_file_open (filename, mu_file_for_reading);
  mu_double_array_read (file, &a2, 0); 
  mu_double_matrix_read (file, &m2, 0);
  mu_double_cube_read (file, &c2, 0);
  mu_file_close (file);

  /* Check if things agree */
  loop_array (i,a1) assert (A_(a1,i) == A_(a2,i));
  loop_matrix (i,j,m1) assert (M_(m1,i,j) == M_(m2,i,j));
  loop_cube (i,j,k,c1) assert (C_(c1,i,j,k) == C_(c2,i,j,k));
  mu_delete (filename);
}

/* 
 * With the variadic versions it is necessary to test for all 3 types 
`*/

/* The double test */
void test_variadic_double (void)
{
  char *filename = "test3_output.dat";
  struct mu_file *file;
  double_array_t a1,a2;  
  double_matrix_t m1,m2;
  double_cube_t c1,c2;
  int i,j,k;
  
  /* Create the arrays */
  a1 = test_make_double_array (); 
  m1 = test_make_double_matrix ();
  c1 = test_make_double_cube ();
  a2 = mu_double_array_alloc (ax);
  m2 = mu_double_matrix_alloc (mx, my);
  c2 = mu_double_cube_alloc (cx, cy, cz);
  
  /* Create the test file */
  file = mu_file_open (filename, mu_file_for_writing);
  mu_file_output (file, "%da %dm %dc", a1, m1, c1);
  mu_file_close (file);

  /* Read the test file */
  file = mu_file_open (filename, mu_file_for_reading);
  mu_file_input (file, "%da %dm %dc", &a2, &m2, &c2);
  mu_file_close (file);

  /* Check if things agree */
  loop_array (i,a1) assert (A_(a1,i) == A_(a2,i));
  loop_matrix (i,j,m1) assert (M_(m1,i,j) == M_(m2,i,j));
  loop_cube (i,j,k,c1) assert (C_(c1,i,j,k) == C_(c2,i,j,k));
  mu_delete (filename);
}

/* The float test */
void test_variadic_float (void)
{
  char *filename = "test3_output.dat";
  struct mu_file *file;
  float_array_t a1,a2;  
  float_matrix_t m1,m2;
  float_cube_t c1,c2;
  int i,j,k;
  
  /* Create the arrays */
  a1 = test_make_float_array (); 
  m1 = test_make_float_matrix ();
  c1 = test_make_float_cube ();
  a2 = mu_float_array_alloc (ax);
  m2 = mu_float_matrix_alloc (mx, my);
  c2 = mu_float_cube_alloc (cx, cy, cz);
  
  /* Create the test file */
  file = mu_file_open (filename, mu_file_for_writing);
  mu_file_output (file, "%fa %fm %fc", a1, m1, c1);
  mu_file_close (file);

  /* Read the test file */
  file = mu_file_open (filename, mu_file_for_reading);
  mu_file_input (file, "%fa %fm %fc", &a2, &m2, &c2);
  mu_file_close (file);

  /* Check if things agree */
  loop_array (i,a1) assert (A_(a1,i) == A_(a2,i));
  loop_matrix (i,j,m1) assert (M_(m1,i,j) == M_(m2,i,j));
  loop_cube (i,j,k,c1) assert (C_(c1,i,j,k) == C_(c2,i,j,k));
  mu_delete (filename);
}

void test_variadic_int (void)
{
  char *filename = "test3_output.dat";
  struct mu_file *file;
  int_array_t a1,a2;  
  int_matrix_t m1,m2;
  int_cube_t c1,c2;
  int i,j,k;
  
  /* Create the arrays */
  a1 = test_make_int_array (); 
  m1 = test_make_int_matrix ();
  c1 = test_make_int_cube ();
  a2 = mu_int_array_alloc (ax);
  m2 = mu_int_matrix_alloc (mx, my);
  c2 = mu_int_cube_alloc (cx, cy, cz);
  
  /* Create the test file */
  file = mu_file_open (filename, mu_file_for_writing);
  mu_file_output (file, "%ia %im %ic", a1, m1, c1);
  mu_file_close (file);

  /* Read the test file */
  file = mu_file_open (filename, mu_file_for_reading);
  mu_file_input (file, "%ia %im %ic", &a2, &m2, &c2);
  mu_file_close (file);

  /* Check if things agree */
  loop_array (i,a1) assert (A_(a1,i) == A_(a2,i));
  loop_matrix (i,j,m1) assert (M_(m1,i,j) == M_(m2,i,j));
  loop_cube (i,j,k,c1) assert (C_(c1,i,j,k) == C_(c2,i,j,k));
  mu_delete (filename);
}

int main ()
{
  test_simple ();
  test_variadic_double ();
  test_variadic_float ();
  test_variadic_int ();
  exit (0);
}
